/*    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package esrpatcher;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;

import util.FileChooser;
import util.ImagePanel;
import util.Utils;

public class Main extends JFrame implements ActionListener, DropTargetListener
{
    static
    {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception e)
        {}

    }

    private static final String URI_LIST_MIME_TYPE = "text/uri-list;class=java.lang.String";

    private ImagePanel btnPatch, btnUnpatch;
    private JFileChooser fileChooser;

    public Main()
    {
        super();

        setTitle("ESR Disc Patcher " + Utils.projectVersion);
        setIconImage(Utils.getImage("resources/img/ico_16x16.png"));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        ImagePanel background = new ImagePanel("resources/img/ESR-DP_03.png");
        background.setPreferredSize(new Dimension(280, 131));
        setContentPane(background);

        JLayeredPane layeredPane = getLayeredPane();

        btnPatch = new ImagePanel(null, "resources/img/ESR-DP_Patch.png", this);
        btnPatch.setBounds(0, 110, 139, 20);
        layeredPane.add(btnPatch, Integer.valueOf(1));

        btnUnpatch = new ImagePanel(null, "resources/img/ESR-DP_Unpatch.png", this);
        btnUnpatch.setBounds(140, 110, 139, 20);
        layeredPane.add(btnUnpatch, Integer.valueOf(1));

        Font labelFont = new Font("Serif", Font.PLAIN, 11);
        JLabel lblInfo1 = new JLabel("Copyright (C) 2008 " + Utils.projectOwner);
        lblInfo1.setAlignmentX(Component.CENTER_ALIGNMENT);
        lblInfo1.setFont(labelFont);
        JLabel lblInfo2 = new JLabel("Great THX goes to berion for the GFX");
        lblInfo2.setAlignmentX(Component.CENTER_ALIGNMENT);
        lblInfo2.setFont(labelFont);
        JLabel lblInfo3 = new JLabel("Either click the \"PATCH\" button or drag and");
        lblInfo3.setAlignmentX(Component.CENTER_ALIGNMENT);
        lblInfo3.setFont(labelFont);
        JLabel lblInfo4 = new JLabel(" drop the files you want to be patched");
        lblInfo4.setAlignmentX(Component.CENTER_ALIGNMENT);
        lblInfo4.setFont(labelFont);

        JPanel container = new JPanel();
        container.setOpaque(false);
        container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
        container.add(lblInfo1);
        container.add(Box.createRigidArea(new Dimension(0, 24)));
        container.add(lblInfo2);
        container.add(Box.createRigidArea(new Dimension(0, 8)));
        container.add(lblInfo3);
        container.add(lblInfo4);
        container.setBounds(0, 3, 280, 100);
        layeredPane.add(container, Integer.valueOf(1));

        pack();

        setResizable(false);
        Utils.centerResizeWindow(this, false);
        setVisible(true);

        new DropTarget(getRootPane(), this);
        fileChooser = new FileChooser();
    }

    public static void main(String args[])
    {
        if (args.length > 0)
        {
            if (args[0].equalsIgnoreCase("-u"))
            {
                List<File> files = Utils.scanInput(args);
                processTxtOutput(files, ESRPatch.unPatch(files));
            }
            else if (args[0].equalsIgnoreCase("-p"))
            {
                List<File> files = Utils.scanInput(args);
                processTxtOutput(files, ESRPatch.patch(files));
            }
            else
            {
                System.err.println("Usage: " + Utils.projectName + " -u|-p file1.iso ...");
            }
        }
        else
        {
            new Main();
        }
    }

    public void actionPerformed(ActionEvent e)
    {
        if (e.getSource() == btnPatch)
        {
            if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
            {
                List<File> files = Arrays.asList(fileChooser.getSelectedFiles());
                List<ESRStatus> results = ESRPatch.patch(files);
                new DialogResult(this, files, results);
            }
        }
        else if (e.getSource() == btnUnpatch)
        {
            if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
            {
                List<File> files = Arrays.asList(fileChooser.getSelectedFiles());
                List<ESRStatus> results = ESRPatch.unPatch(files);
                new DialogResult(this, files, results);
            }
        }
    }

    public void dragEnter(DropTargetDragEvent dtde)
    {}

    public void dragExit(DropTargetEvent dte)
    {}

    public void dragOver(DropTargetDragEvent dtde)
    {}

    public void dropActionChanged(DropTargetDragEvent dtde)
    {}

    @SuppressWarnings("unchecked")
    public void drop(DropTargetDropEvent dtde)
    {
        dtde.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
        try
        {
            List<File> filesList = null;
            DataFlavor uriListFlavor = new DataFlavor(URI_LIST_MIME_TYPE);

            Transferable transferable = dtde.getTransferable();
            if (transferable.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
            {
                List data = (List) transferable.getTransferData(DataFlavor.javaFileListFlavor);
                filesList = new ArrayList<File>(data.size());
                for (Object oneFile : data)
                {
                    filesList.add((File) oneFile);
                }
            }
            else if (transferable.isDataFlavorSupported(uriListFlavor))
            {
                String data = (String) transferable.getTransferData(uriListFlavor);
                filesList = Utils.convertURIListToFileList(data);
            }

            if (filesList != null)
            {
                Object[] options = { "Unpatch", "Patch" };
                int opt = JOptionPane.showOptionDialog(this, "What would you like to do?", "What to do?",
                        JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[1]);

                if (opt == JOptionPane.NO_OPTION)
                {
                    List<ESRStatus> results = ESRPatch.patch(filesList);
                    new DialogResult(this, filesList, results);
                }
                if (opt == JOptionPane.YES_OPTION)
                {
                    List<ESRStatus> results = ESRPatch.unPatch(filesList);
                    new DialogResult(this, filesList, results);
                }
                dtde.dropComplete(true);
            }
            else
            {
                dtde.rejectDrop();
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
            dtde.rejectDrop();
        }
    }
    
    public static void processTxtOutput(List<File> files, List<ESRStatus> results)
    {
        File currFile;
        ESRStatus currResult;
        StringBuilder sb = new StringBuilder();

        Iterator<File> filesIte = files.iterator();
        Iterator<ESRStatus> resultsIte = results.iterator();

        while (filesIte.hasNext())
        {
            currResult = resultsIte.next();
            currFile = filesIte.next();

            if (currResult != null)
            {
                sb.append(currFile.getName()).append(": ").append(currResult.getMessage());
                System.out.println(sb.toString());
                sb.setLength(0);
            }
        }
    }
}
