package opl;

import java.awt.Color;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Properties;

public class Config extends Properties
{
    private static final long serialVersionUID = 1L;

    public static final String PREFS_ETH_PATH = "PREFS_ETH_PATH";
    public static final String PREFS_USB_PATH = "PREFS_USB_PATH";
    public static final String PREFS_PS2_IP = "PREFS_PS2_IP";
    public static final String PREFS_LANGUAGE_FILE = "PREFS_LANGUAGE_FILE";
    public static final String PREFS_FONT_FILE = "PREFS_FONT_FILE";
    public static final String PREFS_APPS_FILE = "PREFS_APPS_FILE";
    public static final String PREFS_DOWNLOAD_URL = "PREFS_DOWNLOAD_URL";
    public static final String PREFS_DISPLAY_GAMEPANEL = "PREFS_DISPLAY_GAMEPANEL";
    
    public Config()
    {}

    public Config(Object file)
    {
        this(file, false);
    }

    public Config(Object file, boolean canFail)
    {
        try
        {
            if (file instanceof String)
            {
                file = new FileInputStream((String) file);
            }
            if (file instanceof InputStream)
            {
                InputStream fileStream = (InputStream) file;
                try
                {
                    load(fileStream);
                }
                finally
                {
                    if (fileStream != null)
                    {
                        fileStream.close();
                    }
                }
            }
            else
            {
                System.err.println("Config: file " + file + " is invalid");
            }
        }
        catch (Exception ex)
        {
            if (!canFail)
            {
                ex.printStackTrace(System.err);
            }
        }
    }

    public void saveConfig(String filename)
    {
        saveConfig(filename, null);
    }
    
    public void saveConfig(String filename, String comments)
    {
        try
        {
            FileOutputStream os = new FileOutputStream(filename);
            try
            {
                this.store(os, comments);
            }
            finally
            {
                os.close();
            }
        }
        catch (Exception ex)
        {
            ex.printStackTrace(System.err);
        }
    }

    public void secureSetProperty(Properties props, String key, String value)
    {
        if (key.length() != 0)
        {
            if (value == null)
            {
                System.err.println("Config: parameter " + key + " set to a null value UNSUPPORTED !");
            }
            else
            {
                props.setProperty(key, value.trim());
            }
        }
    }

    public void setString(String key, String value)
    {
        secureSetProperty(this, key, value);
    }

    public String getString(String key, String _default)
    {
        String value = getProperty(key);
        if (value == null)
        {
            value = _default;
        }
        return value;
    }

    public void setInt(String key, int value)
    {
        secureSetProperty(this, key, Integer.toString(value));
    }

    public int getInt(String key, int _default)
    {
        int res = _default;
        String value = getString(key, null);
        if (value != null)
        {
            try
            {
                res = Integer.parseInt(value);
            }
            catch (Exception ex)
            {
                System.err.println("Config: Integer parameter " + key + " invalid " + value);
            }
        }
        return res;
    }

    public void setColor(String key, Color value)
    {
        secureSetProperty(this, key, value.toString()); // TODO: support Color for OPL conf files
    }

    public Color getColor(String key, Color _default)
    {
        Color res = _default;
        String value = getString(key, null);
        if (value != null)
        {
            try
            {
                res = Color.decode(value);
            }
            catch (Exception ex)
            {
                System.err.println("Config: Color parameter " + key + " invalid " + value);
            }
        }
        return res;
    }

    public void setBoolean(String key, boolean value)
    {
        secureSetProperty(this, key, value ? "1" : "0");
    }

    public boolean getBoolean(String key, boolean _default)
    {
        boolean res = _default;
        String value = getString(key, null);
        if (value != null)
        {
            if (value.equals("0"))
            {
                res = false;
            }
            else
            {
                res = true;
            }
        }
        return res;
    }
}
