package opl;

import static opl.RenderMan.ALIGN_CENTER;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;

import util.Utils;

public class FntSys
{
    private static Font OPLFontInternal = new Font(Font.SANS_SERIF, Font.PLAIN, 16);
    private static Font OPLFont;
    
    static
    {
        OPLFontInternal = loadFont("resources/FreeSans_basic_latin.ttf");
        OPLFont = OPLFontInternal;
    }

    private static float shiftYVal = 1;

    private static int shiftY(int posY)
    {
        return (int) (shiftYVal * posY);
    }

    private static Font loadFont(String path)
    {
        Font result = OPLFontInternal;
        
        if (path != null && path.length() != 0)
        {
            try
            {
                result = Font.createFont(Font.TRUETYPE_FONT, Utils.getResource(path)).deriveFont(Font.PLAIN, 16);
            }
            catch (Exception ex)
            {
                ex.printStackTrace(System.err);
            }
        }
        
        return result;
    }
    
    public static void fntLoad(String path)
    {
        OPLFont = loadFont(path);
    }
    
    public static int fntRenderString(Graphics graphics, int x, int y, int aligned, String string, Color color)
    {
        Dimension dim = fntCalcDimensions(graphics, string);

        if (aligned == ALIGN_CENTER)
        {
            x -= dim.width >> 1;
            y -= dim.height >> 1;
        }

        graphics.setColor(color);
        graphics.drawString(string, x, shiftY(y) + dim.height);

        return dim.width;
    }
    
    public static Dimension fntCalcDimensions(Graphics graphics, String text)
    {
        graphics.setFont(OPLFont); // TODO: something strange with custom language ...
        FontMetrics OPLFontMetrics = graphics.getFontMetrics(OPLFont);
        Dimension dim = new Dimension(OPLFontMetrics.stringWidth(text), OPLFontMetrics.getAscent());
        return dim;
    }
    
    public static void fntSetShiftRatio(float shiftYRatio)
    {
        shiftYVal = shiftYRatio;
    }

    public static void fntResetShiftRatio()
    {
        shiftYVal = 1;
    }
}
