package opl;

import static opl.FntSys.fntRenderString;
import static opl.RenderMan.ALIGN_CENTER;
import static opl.RenderMan.ALIGN_NONE;
import static opl.RenderMan.DIM_UNDEF;
import static opl.RenderMan.MENU_ITEM_HEIGHT;
import static opl.RenderMan.gDefaultCol;
import static opl.RenderMan.rmDrawOverlayPixmap;
import static opl.RenderMan.rmDrawPixmap;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.util.List;

import opl.Themes.ThemeElement;
import oplutil.Elem;
import oplutil.Main;
import oplutil.device.ItemBase;
import oplutil.device.MenuBase;

public class MenuSys
{
    public static Image getCachedTexture(String type, MenuBase menu, ItemBase item, Image defaultImage)
    {
        Image result = menu.getArt(item.startup + type);
        if (result == null)
        {
            result = defaultImage;
        }

        return result;
    }

    public static final Image menuGetCurrentArt(MenuBase menu)
    {
        Image result = null;

        ItemBase cur = menu.getCurrentItem();
        if (cur != null)
        {
            result = getCachedTexture(ItemBase.ART_BACKGROUND, menu, cur, null);
        }

        return result;
    }

    public static void menuDrawStatic(Main main, Graphics graphics, MenuBase menu)
    {
        Themes theme = main.getTheme();

        //// rendering ELEMENT "Menu Icon"
        Image someTex = null, otherTex = null;
        if (theme.menuIcon.enabled)
        {
            someTex = theme.thmGetTexture(menu.elem.icon_id);
            if (someTex != null)
            {
                rmDrawPixmap(graphics, someTex, theme.menuIcon.posX, theme.menuIcon.posY, theme.menuIcon.aligned,
                        theme.menuIcon.width, theme.menuIcon.height, theme.menuIcon.color);
            }
        }

        //// rendering ELEMENT "Menu Text"
        if (theme.menuText.enabled)
        {
            if (menu.getIndex() != 0)
            {
                someTex = theme.thmGetTexture(Textures.LEFT_ICON);
            }
            else
            {
                someTex = null;
            }
            if (menu.getIndex() != 3)
            {
                otherTex = theme.thmGetTexture(Textures.RIGHT_ICON);
            }

            if (theme.menuText.aligned == ALIGN_CENTER)
            {
                int offset = theme.menuText.width >> 1;
                if (someTex != null)
                {
                    rmDrawPixmap(graphics, someTex, theme.menuText.posX - offset, theme.menuText.posY,
                            theme.menuText.aligned, DIM_UNDEF, DIM_UNDEF, gDefaultCol);
                }
                if (otherTex != null)
                {
                    rmDrawPixmap(graphics, otherTex, theme.menuText.posX + offset, theme.menuText.posY,
                            theme.menuText.aligned, DIM_UNDEF, DIM_UNDEF, gDefaultCol);
                }
            }
            else
            {
                if (someTex != null)
                {
                    rmDrawPixmap(graphics, someTex, theme.menuText.posX - someTex.getWidth(null), theme.menuText.posY,
                            theme.menuText.aligned, DIM_UNDEF, DIM_UNDEF, gDefaultCol);
                }
                if (otherTex != null)
                {
                    rmDrawPixmap(graphics, otherTex, theme.menuText.posX + theme.menuText.width, theme.menuText.posY,
                            theme.menuText.aligned, DIM_UNDEF, DIM_UNDEF, gDefaultCol);
                }
            }
            fntRenderString(graphics, theme.menuText.posX, theme.menuText.posY, theme.menuText.aligned, Lang
                    .getI18N(menu.elem.text_id), theme.menuText.color);
        }

        if (menu.getCurrentIndex() != -1)
        {
            ItemBase cur = menu.getCurrentItem();

            //// rendering ELEMENT "Item List"
            ThemeElement itemsList = theme.itemsList;

            int stretchedSize = 0;
            if (theme.itemsListIcons)
            {
                stretchedSize = 20;
                someTex = theme.thmGetTexture(Textures.DISC_ICON);
            }

            int curpos = theme.itemsList.posY;
            int others = 0, curIndex;
            Color color;
            while (others < theme.displayedItems)
            {
                curIndex = menu.getPageStartIndex() + others;
                if (curIndex > menu.getItemsNumber() - 1)
                {
                    break;
                }
                ItemBase ps = menu.getItem(curIndex);

                if (theme.itemsListIcons)
                {
                    otherTex = getCachedTexture(ItemBase.ART_ICON, menu, ps, someTex);
                    if (otherTex != null)
                    {
                        rmDrawPixmap(graphics, otherTex, itemsList.posX, curpos + others * MENU_ITEM_HEIGHT,
                                ALIGN_NONE, stretchedSize, stretchedSize, gDefaultCol);
                    }
                }

                if (curIndex == menu.getCurrentIndex())
                {
                    color = theme.selTextColor;
                }
                else
                {
                    color = theme.itemsList.color;
                }

                fntRenderString(graphics, itemsList.posX + stretchedSize, curpos + others * MENU_ITEM_HEIGHT,
                        ALIGN_NONE, ps.name, color);

                others++;
            }

            //// rendering ELEMENT "Item Cover"
            if (theme.itemCover.enabled)
            {
                otherTex = theme.thmGetTexture(Textures.COVER_OVERLAY);
                if (otherTex != null)
                {
                    someTex = getCachedTexture(ItemBase.ART_COVER, menu, cur, null);
                    if (someTex != null)
                    {
                        rmDrawOverlayPixmap(graphics, otherTex, theme.itemCover.posX, theme.itemCover.posY,
                                theme.itemCover.aligned, theme.itemCover.width, theme.itemCover.height,
                                theme.itemCover.color, someTex, theme.coverBlend_ulx, theme.coverBlend_uly,
                                theme.coverBlend_urx, theme.coverBlend_ury, theme.coverBlend_blx, theme.coverBlend_bly,
                                theme.coverBlend_brx, theme.coverBlend_bry);
                    }
                }
                else
                {
                    someTex = getCachedTexture(ItemBase.ART_COVER, menu, cur, null);
                    if (someTex != null)
                    {
                        rmDrawPixmap(graphics, someTex, theme.itemCover.posX, theme.itemCover.posY,
                                theme.itemCover.aligned, theme.itemCover.width, theme.itemCover.height,
                                theme.itemCover.color);
                    }
                }
            }

            //// rendering ELEMENT "Item Icon"
            if (theme.itemIcon.enabled)
            {
                someTex = getCachedTexture(ItemBase.ART_ICON, menu, cur, null);
                if (someTex != null)
                {
                    rmDrawPixmap(graphics, someTex, theme.itemIcon.posX, theme.itemIcon.posY, theme.itemIcon.aligned,
                            theme.itemIcon.width, theme.itemIcon.height, theme.itemIcon.color);
                }
            }

            //// rendering ELEMENT "Item Text"
            if (theme.itemText.enabled)
            {
                fntRenderString(graphics, theme.itemText.posX, theme.itemText.posY, theme.itemText.aligned,
                        cur.startup, theme.itemText.color);
            }
        }

        //// rendering ELEMENT "Hint Text"
        // single hint
        List<Elem> hints = menu.getHints();

        if (theme.hintText.enabled && hints != null)
        {
            int x = theme.hintText.posX;
            int y = theme.hintText.posY;

            // background
            //rmDrawRect(x, y, ALIGN_NONE, theme.hintText.width, theme.hintText.height, Color.blue);

            for (Elem hint : hints)
            {
                someTex = theme.thmGetTexture(hint.icon_id);
                if (someTex != null)
                {
                    rmDrawPixmap(graphics, someTex, x, y, ALIGN_NONE, DIM_UNDEF, DIM_UNDEF, gDefaultCol);
                    x += someTex.getWidth(null) + 2;
                }

                x += fntRenderString(graphics, x, y, ALIGN_NONE, Lang.getI18N(hint.text_id), theme.hintText.color);
                x += 12;
            }
        }
    }
}
