package opl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;

import oplutil.Main;

class Quad
{
    int ul_x, ul_y, br_x, br_y;
    int ul_u, ul_v, br_u, br_v;
    Color color;
    Image txt;

    public Quad()
    {}
}

public class RenderMan
{

    public static final int DIM_INF = -1;
    public static final int DIM_UNDEF = -2;

    public static final int ALIGN_NONE = 0;
    public static final int ALIGN_CENTER = 1;

    public static final int MENU_ITEM_HEIGHT = 19;

    private static float aspectWidth = 1;
    private static float aspectHeight = 1;

    // Transposition values - all rendering can be transposed (moved on screen) by these
    // this is a post-clipping operation
    private static int transX = 0;
    private static int transY = 0;

    public static Color gDefaultCol = new Color(0x80, 0x80, 0x80);

    private static float shiftYVal = 1;

    private static int shiftY(int posY)
    {
        //return (int) ceil(shiftYVal * posY);
        return (int) (shiftYVal * posY);
    }

    private static void rmDrawQuad(Graphics graphics, Quad q)
    {
        //if (q->txt->PSM == GS_PSM_CT32)
        //    gsKit_set_primalpha(gsGlobal, gDefaultAlpha, 0);

        graphics.drawImage(q.txt, q.ul_x + transX, q.ul_y + transY, q.br_x + transX, q.br_y + transY, q.ul_u, q.ul_v,
                q.br_u, q.br_v, null);

        //gsKit_set_primalpha(gsGlobal, GS_BLEND_BACK2FRONT, 0);
    }

    private static void rmSetupQuad(Image txt, int x, int y, int aligned, int w, int h, Color color, Quad q)
    {
        int drawX, drawY;
        int drawW, drawH;

        if (aligned == ALIGN_CENTER)
        {
            if (w != DIM_UNDEF)
            {
                drawW = (int) (aspectWidth * w);
            }
            else
            {
                drawW = (int) (aspectWidth * txt.getWidth(null));
            }
            drawX = x - drawW / 2;

            if (h != DIM_UNDEF)
            {
                drawH = (int) (aspectHeight * h);
            }
            else
            {
                drawH = (int) (aspectHeight * txt.getHeight(null));
            }
            drawY = shiftY(y) - drawH / 2;
        }
        else
        {
            if (w == DIM_UNDEF)
            {
                drawW = (int) (aspectWidth * txt.getWidth(null));
            }
            else if (w == DIM_INF)
            {
                drawW = Main.screenWidth - x;
            }
            else
            {
                drawW = (int) (aspectWidth * w);
            }
            drawX = x;

            if (h == DIM_UNDEF)
            {
                drawH = (int) (aspectHeight * txt.getHeight(null));
            }
            else if (h == DIM_INF)
            {
                drawH = Main.screenHeight - y;
            }
            else
            {
                drawH = (int) (aspectHeight * h);
            }
            drawY = shiftY(y);
        }

        q.ul_x = drawX;
        q.ul_y = drawY;
        q.br_x = drawX + drawW;
        q.br_y = drawY + drawH;

        q.color = color;

        if (txt != null)
        {
            q.txt = txt;
            q.ul_u = 0;
            q.ul_v = 0;
            q.br_u = txt.getWidth(null);
            q.br_v = txt.getHeight(null);
        }
    }

    public static void rmDrawPixmap(Graphics graphics, Image someTex, int posX, int posY, int aligned, int width,
            int height, Color color)
    {
        Quad quad = new Quad();
        rmSetupQuad(someTex, posX, posY, aligned, width, height, color, quad);
        rmDrawQuad(graphics, quad);
    }

    public static void rmDrawOverlayPixmap(Graphics graphics, Image overlay, int x, int y, int aligned, int w, int h,
            Color color, Image inlay, int ulx, int uly, int urx, int ury, int blx, int bly, int brx, int bry)
    {
        Quad quad = new Quad();
        rmSetupQuad(overlay, x, y, aligned, w, h, color, quad);

        graphics.drawImage(inlay, quad.ul_x + transX + (int) (aspectWidth * ulx), quad.ul_y + transY + uly, quad.ul_x
                + transX + (int) (aspectWidth * brx), quad.ul_y + transY + bry, 0, 0, inlay.getWidth(null), inlay
                .getHeight(null), null);

        /*graphics.drawImage(inlay, quad.ul_x + transX + aspectWidth * ulx, quad.ul_y + transY + uly, 0, 0,
                quad.ul_x + transX + aspectWidth * urx, quad.ul_y + transY + ury, inlay.getWidth(), 0,
                quad.ul_x + transX + aspectWidth * blx, quad.ul_y + transY + bly, 0, inlay.getHeight(),
                quad.ul_x + transX + aspectWidth * brx, quad.ul_y + transY + bry, inlay.getWidth(), inlay.getHeight());*/
        //, order, gDefaultCol);

        //gsKit_set_primalpha(gsGlobal, GS_BLEND_BACK2FRONT, 0);

        rmDrawQuad(graphics, quad);
    }

    public static void rmDrawRect(Graphics graphics, int x, int y, int aligned, int w, int h, Color color)
    {
        Quad quad = new Quad();
        rmSetupQuad(null, x, y, aligned, w, h, color, quad);

        //gsKit_set_primalpha(gsGlobal, GS_SETREG_ALPHA(0,1,0,1,0), 0);
        graphics.setColor(quad.color);
        graphics.fillRect(quad.ul_x + transX, quad.ul_y + transY, quad.br_x - quad.ul_x, quad.br_y - quad.ul_y);
        //gsKit_set_primalpha(gsGlobal, GS_BLEND_BACK2FRONT, 0); 
    }

    public static void rmSetAspectRatio(float width, float height)
    {
        aspectWidth = width;
        aspectHeight = height;
    }

    public static void rmResetAspectRatio()
    {
        aspectWidth = 1.0f;
        aspectHeight = 1.0f;
    }

    /*public static void rmGetAspectRatio(float w, float h)
    {
        *w = aspectWidth;
        *h = aspectHeight;
    }*/

    public static void rmSetShiftRatio(float shiftYRatio)
    {
        shiftYVal = shiftYRatio;
    }

    public static void rmResetShiftRatio()
    {
        shiftYVal = 1;
    }

    public static void rmSetTransposition(float x, float y)
    {
        transX = (int) x;
        transY = (int) y;
    }
}
