package opl;

import static opl.Textures.*;
import static opl.RenderMan.*;
import static opl.FntSys.*;

import java.awt.Color;
import java.awt.Image;

import oplutil.Main;

public class Themes
{
    public static final int BG_MODE_PICTURE = 0;
    public static final int BG_MODE_PLASMA  = 1;
    public static final int BG_MODE_COLOR   = 2;
    public static final int BG_MODE_ART     = 3;
    
    public static Color gDefaultBgColor = new Color(0x028, 0x0c5, 0x0f9);
    public static Color gDefaultTextColor = new Color(0x0ff, 0x0ff, 0x0ff);
    public static Color gDefaultSelTextColor = new Color(0x0ff, 0x080, 0x00);
    public static Color gDefaultUITextColor = new Color(0x040, 0x080, 0x040);
    
    public static final int MENU_POS_V = 50;
    public static final int HINT_HEIGHT = 32;

    ////////////////////////////////////////////////////////////////////////////////////////////////    

    class ThemeElement
    {
        public String name;
        public boolean enabled;
        public int posX;
        public int posY;
        public int aligned;
        public int width;
        public int height;
        public Color color;
        
        public ThemeElement()
        {}
    }

    public boolean useDefault;
    public int usedHeight;
    public int displayedItems;
    public boolean itemsListIcons;

    public Color bgColor;
    public Color textColor;
    public Color uiTextColor;
    public Color selTextColor;

    public ThemeElement menuIcon;
    public ThemeElement menuText;
    public ThemeElement itemsList;
    public ThemeElement itemIcon;
    public ThemeElement itemCover;
    public ThemeElement itemText;
    public ThemeElement hintText;
    public ThemeElement loadingIcon;
    public int busyIconsCount;

    public int coverBlend_ulx, coverBlend_uly, coverBlend_urx, coverBlend_ury;
    public int coverBlend_blx, coverBlend_bly, coverBlend_brx, coverBlend_bry;

    public Image[] textures;

    public int drawBackground;
    
    public Themes()
    {
        setColors();

        menuIcon = new ThemeElement();
        menuText = new ThemeElement();
        itemsList = new ThemeElement();
        itemIcon = new ThemeElement();
        itemCover = new ThemeElement();
        itemText = new ThemeElement();
        hintText = new ThemeElement();
        loadingIcon = new ThemeElement();

        textures = new Image[Textures.TEXTURES_COUNT];
    }

    /*private static final int PLASMA_H = 16;
    private static final int PLASMA_W = 16;
    private static final int PLASMA_ROWS_PER_FRAME = 16;

    private int pperm[] = new int[512];
    float pgrad3[][] = {
        {1,1,0},{-1,1,0},{1,-1,0},{-1,-1,0}, 
        {1,0,1},{-1,0,1},{1,0,-1},{-1,0,-1}, 
        {0,1,1},{0,-1,1},{0,1,-1},{0,-1,-1}
    };
    
    float dot(float g[], float x, float y, float z)
    { 
        return g[0]*x + g[1]*y + g[2]*z; 
    }

    float mix(float a, float b, float t)
    { 
        return (1.0f-t)*a + t*b; 
    }

    float fade(float t)
    { 
        return t*t*t*(t*(t*6.0f-15.0f)+10.0f); 
    }


    float guiCalcPerlin(float x, float y, float z) {
        // Taken from: http://people.opera.com/patrickl/experiments/canvas/plasma/perlin-noise-classical.js
        // By Sean McCullough
        
        // Find unit grid cell containing point 
        int X = (int) Math.floor(x); 
        int Y = (int) Math.floor(y); 
        int Z = (int) Math.floor(z); 

        // Get relative xyz coordinates of point within that cell 
        x = x - X; 
        y = y - Y; 
        z = z - Z; 

        // Wrap the integer cells at 255 (smaller integer period can be introduced here) 
        X = X & 255; 
        Y = Y & 255; 
        Z = Z & 255;

        // Calculate a set of eight hashed gradient indices 
        int gi000 = pperm[X+pperm[Y+pperm[Z]]] % 12; 
        int gi001 = pperm[X+pperm[Y+pperm[Z+1]]] % 12; 
        int gi010 = pperm[X+pperm[Y+1+pperm[Z]]] % 12; 
        int gi011 = pperm[X+pperm[Y+1+pperm[Z+1]]] % 12; 
        int gi100 = pperm[X+1+pperm[Y+pperm[Z]]] % 12; 
        int gi101 = pperm[X+1+pperm[Y+pperm[Z+1]]] % 12; 
        int gi110 = pperm[X+1+pperm[Y+1+pperm[Z]]] % 12; 
        int gi111 = pperm[X+1+pperm[Y+1+pperm[Z+1]]] % 12; 

        // The gradients of each corner are now: 
        // g000 = grad3[gi000]; 
        // g001 = grad3[gi001]; 
        // g010 = grad3[gi010]; 
        // g011 = grad3[gi011]; 
        // g100 = grad3[gi100]; 
        // g101 = grad3[gi101]; 
        // g110 = grad3[gi110]; 
        // g111 = grad3[gi111]; 
        // Calculate noise contributions from each of the eight corners 
        float n000 = dot(pgrad3[gi000], x, y, z); 
        float n100 = dot(pgrad3[gi100], x-1, y, z); 
        float n010 = dot(pgrad3[gi010], x, y-1, z); 
        float n110 = dot(pgrad3[gi110], x-1, y-1, z); 
        float n001 = dot(pgrad3[gi001], x, y, z-1); 
        float n101 = dot(pgrad3[gi101], x-1, y, z-1); 
        float n011 = dot(pgrad3[gi011], x, y-1, z-1); 
        float n111 = dot(pgrad3[gi111], x-1, y-1, z-1); 
        
        // Compute the fade curve value for each of x, y, z 
        float u = fade(x); 
        float v = fade(y); 
        float w = fade(z); 
        
        // Interpolate along x the contributions from each of the corners 
        float nx00 = mix(n000, n100, u); 
        float nx01 = mix(n001, n101, u); 
        float nx10 = mix(n010, n110, u); 
        float nx11 = mix(n011, n111, u); 
        // Interpolate the four results along y 
        float nxy0 = mix(nx00, nx10, v); 
        float nxy1 = mix(nx01, nx11, v); 
        // Interpolate the two last results along z 
        float nxyz = mix(nxy0, nxy1, w); 

        return nxyz; 
    }

    float dir = 0.03f;
    float perz = -100f;
    int pery = 0;
    
    void guiDrawBackground()
    {
        int x;
        int y;
        
        // it's PLASMA_ROWS_PER_FRAME rows a frame to stop being a resource hog
        if (pery >= PLASMA_H)
        {
            pery = 0;
            perz += dir;
            
            if (perz > 100.0f || perz < -100.0f)
                dir = -dir;
        }
        
        //u32 *buf = gBackgroundTex.Mem + PLASMA_W * pery;
        int ymax = pery + PLASMA_ROWS_PER_FRAME;
        
        if (ymax > PLASMA_H)
            ymax = PLASMA_H;
        
        for (y = pery; y < ymax; y++)
        {
            for (x = 0; x < PLASMA_W; x++)
            {
                float fper = guiCalcPerlin((float)(x) / PLASMA_W, (float)(y) / PLASMA_H, perz) * 0x080 + 0x080;
                
                *buf = GS_SETREG_RGBA(
                    (u32)(fper * gDefaultBgColor[0]) >> 8,
                    (u32)(fper * gDefaultBgColor[1]) >> 8, 
                    (u32)(fper * gDefaultBgColor[2]) >> 8, 
                    0x080);
                
                ++buf;
            }
        }
        
        pery = ymax;
        rmDrawStretchedPixmap(0, 0, Main.screenWidth, Main.screenHeight, &gBackgroundTex);
    }*/

    public Image thmGetTexture(int id)
    {
        if (id >= Textures.TEXTURES_COUNT)
        {
            return null;
        }
        else
        {
            // see if the texture is valid
            Image txt = textures[id];

            if (txt != null)
            {
                return txt;
            }
            else
            {
                return null;
            }
        }
    }

    public boolean thmLoadResource(int texId, String themePath)
    {
        Image texture = null;

        if (themePath != null)
        {
            texture = Textures.texDiscoverLoad(themePath, texId); // only set success here
            textures[texId] = texture;
        }

        if ((texture == null) && useDefault)
        {
            textures[texId] = Textures.texDiscoverLoad(null, texId);
        }

        return texture != null;
    }

    public void getElem(Config themeConfig, ThemeElement elem, String name, boolean enabled, int x, int y, int aligned,
            int w, int h, boolean defColor)
    {
        int intValue;

        elem.name = name;

        elem.enabled = themeConfig.getBoolean(name + "_enabled", enabled);

        intValue = themeConfig.getInt(name + "_x", x);
        if (intValue < 0)
            elem.posX = Main.screenWidth + intValue;
        else
            elem.posX = intValue;

        intValue = themeConfig.getInt(name + "_y", y);
        if (intValue < 0)
            elem.posY = (int) Math.ceil((Main.screenHeight + intValue) * usedHeight / Main.screenHeight);
        else
            elem.posY = intValue;

        elem.aligned = themeConfig.getInt(name + "_aligned", aligned);

        intValue = themeConfig.getInt(name + "_width", w);
        elem.width = intValue;
        if (intValue < 0)
        {
            if (intValue == DIM_INF)
            {
                if (elem.aligned == ALIGN_CENTER) // incompatible, so set to unaligned
                    elem.aligned = ALIGN_NONE;
            }
        }

        intValue = themeConfig.getInt(name + "_height", h);
        elem.height = intValue;
        if (intValue < 0)
        {
            if (intValue == DIM_INF)
            {
                if (elem.aligned == ALIGN_CENTER) // incompatible, so reset to unaligned
                    elem.aligned = ALIGN_NONE;
            }
        }

        if (defColor)
        {
            elem.color = gDefaultCol;
        }
        else
        {
            elem.color = themeConfig.getColor(name + "_color", textColor); // TODO: alpha 0xff ?
        }
    }

    public void setColors()
    {
        bgColor = gDefaultBgColor;
        textColor = gDefaultTextColor;
        uiTextColor = gDefaultUITextColor;
        selTextColor = gDefaultSelTextColor;
    }

    public void thmLoad(String themePath)
    {
        Config themeConfig;
        if (themePath != null)
        {
            themeConfig = new Config(themePath + "conf_theme.cfg", true); // try to load the theme config file
        }
        else
        {
            themeConfig = new Config();
        }

        int intValue;
        useDefault = themeConfig.getBoolean("use_default", true);
        itemsListIcons = themeConfig.getBoolean("items_list_icons", false);

        if (themeConfig.getBoolean("use_real_height", false))
        {
            usedHeight = Main.screenHeight;
        }
        else
        {
            usedHeight = 480;
        }

        displayedItems = (usedHeight - (MENU_POS_V + HINT_HEIGHT)) / MENU_ITEM_HEIGHT;
        intValue = themeConfig.getInt("displayed_items", Integer.MAX_VALUE);
        if (intValue < displayedItems)
        {
            displayedItems = intValue;
        }

        drawBackground = themeConfig.getInt("background_mode", BG_MODE_PICTURE); 
            
        bgColor = themeConfig.getColor("bg_color", gDefaultBgColor);
        textColor = themeConfig.getColor("text_color", gDefaultTextColor);
        uiTextColor = themeConfig.getColor("ui_text_color", gDefaultTextColor);
        selTextColor = themeConfig.getColor("sel_text_color", gDefaultSelTextColor);

        getElem(themeConfig, menuIcon, "menu_icon", true, 40, 40, ALIGN_CENTER, DIM_UNDEF, DIM_UNDEF, true);
        getElem(themeConfig, menuText, "menu_text", true, Main.screenWidth >> 1, 20, ALIGN_CENTER, 200, 20, false);
        getElem(themeConfig, itemsList, "items_list", true, 150, MENU_POS_V, ALIGN_NONE, DIM_INF, displayedItems
                * MENU_ITEM_HEIGHT, false);
        getElem(themeConfig, itemIcon, "item_icon", true, 80, usedHeight >> 1, ALIGN_CENTER, DIM_UNDEF, DIM_UNDEF, true);
        getElem(themeConfig, itemCover, "item_cover", true, 520, usedHeight >> 1, ALIGN_CENTER, DIM_UNDEF, DIM_UNDEF,
                true);
        getElem(themeConfig, itemText, "item_text", true, 520, 370, ALIGN_CENTER, DIM_UNDEF, 20, false);
        getElem(themeConfig, hintText, "hint_text", true, 16, -HINT_HEIGHT, ALIGN_NONE, DIM_INF, HINT_HEIGHT, false);
        getElem(themeConfig, loadingIcon, "loading_icon", true, -50, -50, ALIGN_CENTER, DIM_UNDEF, DIM_UNDEF, true);

        coverBlend_ulx = themeConfig.getInt("cover_blend_ulx", 0);
        coverBlend_uly = themeConfig.getInt("cover_blend_uly", 0);
        coverBlend_urx = themeConfig.getInt("cover_blend_urx", 0);
        coverBlend_ury = themeConfig.getInt("cover_blend_ury", 0);
        coverBlend_blx = themeConfig.getInt("cover_blend_blx", 0);
        coverBlend_bly = themeConfig.getInt("cover_blend_bly", 0);
        coverBlend_brx = themeConfig.getInt("cover_blend_brx", 0);
        coverBlend_bry = themeConfig.getInt("cover_blend_bry", 0);

        busyIconsCount = 8;

        if (usedHeight == Main.screenHeight)
        {
            rmResetShiftRatio();
            fntResetShiftRatio();
        }
        else
        {
            rmSetShiftRatio((float) Main.screenHeight / usedHeight);
            fntSetShiftRatio((float) Main.screenHeight / usedHeight);
        }

        int i;

        // First start with busy icon
        String path = themePath;
        boolean customBusy = false;
        for (i = LOAD0_ICON; i <= LOAD7_ICON; i++)
        {
            if (thmLoadResource(i, path))
            {
                customBusy = true;
            }
            else
            {
                if (customBusy)
                {
                    break;
                }
                else
                {
                    path = null;
                }
            }
        }
        busyIconsCount = i;

        // Customizable icons
        for (i = EXIT_ICON; i <= DOWN_ICON; i++)
        {
            thmLoadResource(i, themePath);
        }

        // Not  customizable icons
        for (i = L1_ICON; i <= START_ICON; i++)
        {
            thmLoadResource(i, null);
        }

        // Finish with background
        thmLoadResource(COVER_OVERLAY, themePath);
        thmLoadResource(BACKGROUND_PICTURE, themePath);

        // LOGO is hardcoded
        thmLoadResource(LOGO_PICTURE, null);
    }
}
