package oplutil;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import oplutil.device.ItemBase;
import oplutil.device.MenuBase;
import util.Utils;
import esrpatcher.ESRStatus;

public class GamePanel extends JPanel
{
    private static final long serialVersionUID = 1L;

    public static final String ACTION_ESR = "esr_action";
    public static final String ACTION_MEDIA = "media_action";
    public static final String ACTION_FORMAT = "format_action";

    private static Icon esrOn, esrOff, mediaDVD, mediaCD, formatISO, formatUL, statusError, statusNeutral;

    static
    {
        esrOn = new ImageIcon(Utils.getImage("resources/img/esr_on.png"));
        esrOff = new ImageIcon(Utils.getImage("resources/img/esr_off.png"));
        formatISO = new ImageIcon(Utils.getImage("resources/img/format_iso.png"));
        formatUL = new ImageIcon(Utils.getImage("resources/img/format_ul.png"));
        mediaCD = new ImageIcon(Utils.getImage("resources/img/media_cd.png"));
        mediaDVD = new ImageIcon(Utils.getImage("resources/img/media_dvd.png"));
        statusNeutral = new ImageIcon(Utils.getImage("resources/img/neutral.png"));
        statusError = new ImageIcon(Utils.getImage("resources/img/error.png"));
    }

    private JLabel startupText, sizeText, partsText, fullText;
    private JTextField gameText;
    private JLabel startupLabel, sizeLabel, partsLabel;
    private JButton esrLabelIcon, mediaLabelIcon, formatLabelIcon;

    public GamePanel(ActionListener screen)
    {
        super();

        setBorder(BorderFactory.createLoweredBevelBorder());
        setLayout(new GridBagLayout());

        gameText = new JTextField("Game");
        add(gameText, new GridBagConstraints(0, 0, 10, 1, 1.0, 0.0, GridBagConstraints.NORTH,
                GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));

        startupLabel = new JLabel("Startup");
        startupLabel.setPreferredSize(new Dimension(100, 20));
        startupLabel.setHorizontalTextPosition(SwingConstants.LEADING);
        add(startupLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        startupText = new JLabel();
        add(startupText, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

        sizeLabel = new JLabel("Size");
        sizeLabel.setHorizontalTextPosition(SwingConstants.LEADING);
        add(sizeLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(0, 5, 0, 0), 0, 0));
        sizeText = new JLabel();
        add(sizeText, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

        partsLabel = new JLabel("Parts");
        partsLabel.setHorizontalTextPosition(SwingConstants.LEADING);
        add(partsLabel, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
                new Insets(0, 5, 5, 0), 0, 0));
        partsText = new JLabel();
        add(partsText, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));

        fullText = new JLabel("Full name");
        fullText.setForeground(Color.GRAY);
        add(fullText, new GridBagConstraints(0, 4, 10, 1, 1.0, 0.0, GridBagConstraints.SOUTH,
                GridBagConstraints.HORIZONTAL, new Insets(0, 5, 5, 5), 0, 0));

        mediaLabelIcon = new JButton(mediaCD);
        mediaLabelIcon.addActionListener(screen);
        mediaLabelIcon.setActionCommand(ACTION_MEDIA);
        add(mediaLabelIcon, new GridBagConstraints(2, 1, 1, 3, 0.0, 0.0, GridBagConstraints.EAST,
                GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));

        esrLabelIcon = new JButton(esrOff);
        esrLabelIcon.addActionListener(screen);
        esrLabelIcon.setActionCommand(ACTION_ESR);
        add(esrLabelIcon, new GridBagConstraints(3, 1, 1, 3, 0.0, 0.0, GridBagConstraints.EAST,
                GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));

        formatLabelIcon = new JButton(formatUL);
        formatLabelIcon.addActionListener(screen);
        formatLabelIcon.setActionCommand(ACTION_FORMAT);
        add(formatLabelIcon, new GridBagConstraints(4, 1, 1, 3, 0.0, 0.0, GridBagConstraints.EAST,
                GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
    }

    public void initItem(MenuBase menu, ItemBase item)
    {
        if (item == null)
        {
            return;
        }

        fullText.setText(item.leadingName);

        if (!item.isValid)
        {
            gameText.setForeground(Color.RED);
            gameText.setText("Error reading file on disk");

            startupLabel.setIcon(null);
            startupLabel.setToolTipText(null);
            startupText.setText(null);
            startupText.setForeground(Color.BLACK);

            sizeLabel.setIcon(null);
            sizeLabel.setToolTipText(null);
            sizeText.setText(null);
            sizeText.setForeground(Color.BLACK);

            partsLabel.setIcon(null);
            partsLabel.setToolTipText(null);
            partsText.setText(null);
            partsText.setForeground(Color.BLACK);
        }
        else
        {
            // name
            gameText.setText(item.name);
            gameText.setForeground(Color.BLACK);

            // startup
            startupText.setText(item.startup);
            if (item.realStartup == null)
            {
                if (item.isISO)
                {
                    startupLabel.setIcon(statusError);
                    startupLabel.setToolTipText("Could not extract game code from ISO");
                    startupText.setForeground(Color.RED);
                }
                else
                {
                    startupLabel.setIcon(statusNeutral);
                    startupLabel.setToolTipText("Could not extract game code from 1st file part");
                    startupText.setForeground(Color.ORANGE);
                }
            }
            else
            {
                if (!item.realStartup.equals(item.startup))
                {
                    startupLabel.setIcon(statusError);
                    startupLabel.setToolTipText("Game code in ISO (" + item.realStartup + ") is different");
                    startupText.setForeground(Color.RED);
                    startupText.setToolTipText("Game code in ISO (" + item.realStartup + ") is different");
                }
                else
                {
                    startupLabel.setIcon(null);
                    startupLabel.setToolTipText(null);
                    startupText.setForeground(Color.BLACK);
                }
            }

            // size
            sizeText.setText("" + (item.size / 1024 / 1024) + " Mb");
            if (item.size == 0)
            {
                sizeLabel.setIcon(statusError);
                sizeLabel.setToolTipText("Game file on disk is empty");
                sizeText.setForeground(Color.RED);
            }
            else if ((item.media == 0x12) && (item.size > 900 * 1024 * 1024))
            {
                sizeLabel.setIcon(statusNeutral);
                sizeLabel.setToolTipText("Game type is CD, but size is over 700 Mb");
                sizeText.setForeground(Color.ORANGE);
            }
            else
            {
                sizeLabel.setIcon(null);
                sizeLabel.setToolTipText(null);
                sizeText.setForeground(Color.BLACK);
            }

            // parts
            partsText.setText("" + item.parts);
            if (item.realParts != item.parts)
            {
                partsLabel.setIcon(statusError);
                partsLabel.setToolTipText("Game parts number on disk (" + item.realParts + ") is different");
                partsText.setForeground(Color.RED);
            }
            else
            {
                partsLabel.setIcon(null);
                partsLabel.setToolTipText(null);
                partsText.setForeground(Color.BLACK);
            }
        }

        // media type
        if (item.media == 0x12)
        {
            mediaLabelIcon.setIcon(mediaCD);
            mediaLabelIcon.setToolTipText("Click to change media to DVD");
        }
        else
        {
            mediaLabelIcon.setIcon(mediaDVD);
            mediaLabelIcon.setToolTipText("Click to change media to CD");
        }

        // format
        if (item.isISO)
        {
            formatLabelIcon.setIcon(formatISO);
            formatLabelIcon.setToolTipText(null);
        }
        else
        {
            formatLabelIcon.setIcon(formatUL);
            if (menu.isFat32() && (item.size > 4 * 1024 * 1024 * 1024))
            {
                formatLabelIcon.setToolTipText(null);
            }
            else
            {
                formatLabelIcon.setToolTipText("Click to convert format to ISO");
            }
        }

        // esr status
        if (item.esrStatus == ESRStatus.WARN_ALREADY_PATCHED)
        {
            esrLabelIcon.setEnabled(true);
            esrLabelIcon.setIcon(esrOn);
            esrLabelIcon.setToolTipText("Click to unpatch");
        }
        else if (item.esrStatus == ESRStatus.WARN_NOT_PATCHED)
        {
            esrLabelIcon.setEnabled(true);
            esrLabelIcon.setIcon(esrOff);
            esrLabelIcon.setToolTipText("Click to patch");
        }
        else
        {
            esrLabelIcon.setEnabled(false);
            esrLabelIcon.setIcon(esrOff);
            esrLabelIcon.setToolTipText(null);
        }
    }
}
