package oplutil;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.UIManager;

import opl.Config;
import opl.FntSys;
import opl.Lang;
import opl.Textures;
import opl.Themes;
import oplutil.device.MenuApp;
import oplutil.device.MenuBase;
import oplutil.device.MenuHdd;
import util.Utils;

public class Main extends JFrame
{
    private static final long serialVersionUID = 1L;
    
    public static final int MENU_USB = 0;;
    public static final int MENU_ETH = 1;
    public static final int MENU_HDD = 2;
    public static final int MENU_APP = 3;

    public static int screenWidth = 640;
    public static int screenHeight = 512;

    static
    {
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        catch (Exception e)
        {}
    }

    public static Main main;

    public static void main(String[] args)
    {
        main = new Main();
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////

    private MenuBar menuBar;
    private Screen screen;
    private SettingsPanel settingsPanel;
    private GamePanel gamePanel;
    private List<MenuBase> menus = new ArrayList<MenuBase>();
    private Themes theme;
    private Config prefs;

    public Main()
    {
        super();
        
        String userHome = System.getProperty("user.home");
        prefs = new Config(userHome + "/." + Utils.projectName.toLowerCase(), true);

        menus.add(new MenuBase(this, MENU_USB, Config.PREFS_USB_PATH, true, Lang._STR_USB_GAMES, Textures.USB_ICON));
        menus.add(new MenuBase(this, MENU_ETH, Config.PREFS_ETH_PATH, false, Lang._STR_NET_GAMES, Textures.ETH_ICON));
        menus.add(new MenuHdd(this, MENU_HDD, Config.PREFS_PS2_IP, Lang._STR_HDD_GAMES, Textures.HDD_ICON));
        menus.add(new MenuApp(this, MENU_APP, Config.PREFS_APPS_FILE, Lang._STR_APPS, Textures.APP_ICON));

        setTitle(Utils.projectName + ' ' + Utils.projectVersion);
        setIconImage(Utils.getImage("resources/img/ico_16x16.png"));
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        boolean displayGamePanel = prefs.getBoolean(Config.PREFS_DISPLAY_GAMEPANEL, false);
        menuBar = new MenuBar(this, displayGamePanel);
        setJMenuBar(menuBar);
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                String userHome = System.getProperty("user.home");
                prefs.saveConfig(userHome + "/." + Utils.projectName.toLowerCase());
            }
        });

        getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));

        screen = new Screen(this);
        getContentPane().add(screen);

        settingsPanel = new SettingsPanel(this);
        getContentPane().add(settingsPanel);

        gamePanel = new GamePanel(screen);
        toggleGamePanelState(displayGamePanel);
        
        init();
        
        pack();

        setResizable(false);
        Utils.centerResizeWindow(this, false);
        setVisible(true);

        screen.requestFocusInWindow();
    }
    
    public void init()
    {
        menus.get(0).refreshMenu(false);
        menus.get(1).refreshMenu(false);
        menus.get(2).refreshMenu(false);
        menus.get(3).refreshMenu(false);
        
        FntSys.fntLoad(prefs.getString(Config.PREFS_FONT_FILE, null));
        Lang.lngLoadFromFile(prefs.getString(Config.PREFS_LANGUAGE_FILE, null));
        
        settingsPanel.reloadThemes();
    }

    public Screen getScreen()
    {
        return screen;
    }

    public SettingsPanel getInfoPanel()
    {
        return settingsPanel;
    }

    public GamePanel getGamePanel()
    {
        return gamePanel;
    }

    public MenuBase getMenu(int index)
    {
        return menus.get(index);
    }

    public Themes getTheme()
    {
        return theme;
    }

    public boolean setTheme(Themes newTheme)
    {
        boolean needRefresh = false;
        if (theme != null)
        {
            needRefresh = theme.displayedItems != newTheme.displayedItems;
        }
        
        theme = newTheme;
        
        return needRefresh;
    }

    public Config getPrefs()
    {
        return prefs;
    }
    
    public void toggleGamePanelState(boolean add)
    {
        if (add)
        {
            getContentPane().add(gamePanel);
        }
        else
        {
            getContentPane().remove(gamePanel);
        }
        pack();
        Utils.centerResizeWindow(this, false);
    }
    
    public boolean isGamePanelDisplayed()
    {
        return menuBar.displayGamePanelState();
    }
}
