package oplutil;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import opl.Gui;
import opl.MenuSys;
import opl.Themes;
import oplutil.device.ItemBase;
import oplutil.device.MenuBase;
import util.Utils;
import esrpatcher.ESRStatus;

public class Screen extends JPanel implements KeyListener, MouseListener, ActionListener
{
    private static final long serialVersionUID = 1L;

    private Main main;
    private MenuBase menu;
    private boolean wideScreen;

    public Screen(Main parent)
    {
        super();
        main = parent;
        menu = parent.getMenu(Main.MENU_USB);
        setWideScreen(false);
        addKeyListener(this);
        addMouseListener(this);
    }

    @Override
    public void paintComponent(Graphics g)
    {
        Graphics2D g2d = (Graphics2D) g;
        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_GASP);
        //g2d.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_SPEED);

        g2d.scale(wideScreen ? 4f / 3f : 1, 480f / Main.screenHeight);

        int bgMode = main.getTheme().drawBackground;
        if (bgMode == Themes.BG_MODE_ART)
        {
            Gui.guiDrawBGArt(main, g, menu);
        }
        else if (bgMode == Themes.BG_MODE_PICTURE)
        {
            Gui.guiDrawBGPicture(main, g, menu);
        }
        else if (bgMode == Themes.BG_MODE_PLASMA)
        {
            Gui.guiDrawBGPlasma(main, g, menu);
        }
        else if (bgMode == Themes.BG_MODE_COLOR)
        {
            Gui.guiDrawBGColor(main, g, menu);
        }
        MenuSys.menuDrawStatic(main, g, menu);
    }

    public void init()
    {
        // TODO main.getGamePanel().initItem()
        repaint();
    }
    
    public void setWideScreen(boolean newValue)
    {
        wideScreen = newValue;

        if (newValue)
        {
            Dimension dim = new Dimension((int) (640 * 4f / 3f), 480);
            setPreferredSize(dim);
            setSize(dim);
        }
        else
        {
            Dimension dim = new Dimension(640, 480);
            setPreferredSize(dim);
            setSize(dim);
        }

        main.pack();
        Utils.centerResizeWindow(main, false);
    }

    @Override
    public void keyPressed(KeyEvent e)
    {}

    @Override
    public void keyReleased(KeyEvent e)
    {
        int vk = e.getKeyCode();

        if (vk == KeyEvent.VK_DOWN)
        {
            if (menu.getCurrentIndex() < menu.getItemsNumber() - 1)
            {
                menu.setCurrentIndex(menu.getCurrentIndex() + 1);
            }

            // verify the item is in visible range
            if (menu.getCurrentIndex() > (menu.getPageStartIndex() + main.getTheme().displayedItems - 1))
            {
                menu.setPageStartIndex(menu.getCurrentIndex());
            }

            repaint();
        }
        else if (vk == KeyEvent.VK_UP)
        {
            if (menu.getCurrentIndex() > 0)
            {
                menu.setCurrentIndex(menu.getCurrentIndex() - 1);
            }

            // verify the item is in visible range
            if (menu.getCurrentIndex() < menu.getPageStartIndex())
            {
                menu.setPageStartIndex(menu.getPageStartIndex() - main.getTheme().displayedItems);
            }

            repaint();
        }
        else if (vk == KeyEvent.VK_LEFT)
        {
            if (menu.getIndex() != 0)
            {
                menu = main.getMenu(menu.getIndex() - 1);
                repaint();
            }
        }
        else if (vk == KeyEvent.VK_RIGHT)
        {
            if (menu.getIndex() != 3)
            {
                menu = main.getMenu(menu.getIndex() + 1);
                repaint();
            }
        }
        else if (vk == KeyEvent.VK_I)
        {
            if (!main.isGamePanelDisplayed())
            {
                new GameDialog(main);
            }
        }
    }

    @Override
    public void keyTyped(KeyEvent e)
    {}

    @Override
    public void mouseClicked(MouseEvent e)
    {
        requestFocusInWindow();
    }

    @Override
    public void mouseEntered(MouseEvent e)
    {}

    @Override
    public void mouseExited(MouseEvent e)
    {}

    @Override
    public void mousePressed(MouseEvent e)
    {}

    @Override
    public void mouseReleased(MouseEvent e)
    {}

    @Override
    public void actionPerformed(ActionEvent e)
    {
        ItemBase item = menu.getCurrentItem();
        if (item.isValid || true)
        {
            String finalCommand = e.getActionCommand();
            if (finalCommand == GamePanel.ACTION_ESR)
            {
                String message;
                if (item.esrStatus == ESRStatus.WARN_ALREADY_PATCHED)
                {
                    message = "This will unpatch ESR, continue ?";
                }
                else
                {
                    message = "This will apply ESR, continue ?";
                }
                if (JOptionPane.showConfirmDialog(this, message, "ESR Action", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
                {
                    menu.changeESR(item);
                    main.getGamePanel().initItem(menu, item);
                }
            }
            else if (finalCommand == GamePanel.ACTION_FORMAT)
            {

            }
            else if (finalCommand == GamePanel.ACTION_MEDIA)
            {
                String message;
                if (item.media == 0x12)
                {
                    message = "This will change the media type to DVD, continue ?";
                }
                else
                {
                    message = "This will change the media type to CD, continue ?";
                }
                if (JOptionPane.showConfirmDialog(this, message, "Media Action", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
                {
                    menu.changeMediaType(item);
                    main.getScreen().repaint();
                    //main.getGamePanel().initItem(menu, item);
                }
            }
        }
    }
}
