package oplutil;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import opl.Config;
import util.Utils;

public class SettingsDialog extends JDialog implements ActionListener
{
    private static final long serialVersionUID = 1L;

    private static final String EMPTY_STRING = "";

    private Main main;

    private JTextField ethPathTextField, usbPathTextField, PS2IPTextField, appsTextField;
    private JTextField languageTextField, fontTextField, downloadTextField;
    //private JCheckBox simulateTVRendering;
    private JButton okButton;

    public SettingsDialog(Main parent, String title)
    {
        super(parent, title, true);
        main = parent;

        JRootPane rootPane = getRootPane();
        rootPane.setLayout(new BorderLayout());

        JPanel centerPanel = new JPanel();
        centerPanel.setLayout(new GridBagLayout());

        JLabel ethPathLabel = new JLabel("PS2SMB share: ");
        centerPanel.add(ethPathLabel, new GridBagConstraints(0, 0, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(5, 5, 0, 0), 0, 0));
        ethPathTextField = new JTextField(main.getPrefs().getString(Config.PREFS_ETH_PATH, EMPTY_STRING));
        centerPanel.add(ethPathTextField, new GridBagConstraints(1, 0, 1, 1, 0.80, 0.0, GridBagConstraints.NORTH,
                GridBagConstraints.HORIZONTAL, new Insets(5, 0, 0, 5), 0, 0));

        JLabel usbPathLabel = new JLabel("USB device path: ");
        centerPanel.add(usbPathLabel, new GridBagConstraints(0, 1, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        usbPathTextField = new JTextField(main.getPrefs().getString(Config.PREFS_USB_PATH, EMPTY_STRING));
        centerPanel.add(usbPathTextField, new GridBagConstraints(1, 1, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 5), 0, 0));

        JLabel PS2IPLabel = new JLabel("PS2 IP: ");
        centerPanel.add(PS2IPLabel, new GridBagConstraints(0, 2, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        PS2IPTextField = new JTextField(main.getPrefs().getString(Config.PREFS_PS2_IP, EMPTY_STRING));
        centerPanel.add(PS2IPTextField, new GridBagConstraints(1, 2, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 5), 0, 0));

        JLabel AppsLabel = new JLabel("Applications file: ");
        centerPanel.add(AppsLabel, new GridBagConstraints(0, 3, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        appsTextField = new JTextField(main.getPrefs().getString(Config.PREFS_APPS_FILE, EMPTY_STRING));
        centerPanel.add(appsTextField, new GridBagConstraints(1, 3, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 5), 0, 0));

        JLabel languageLabel = new JLabel("Language file: ");
        centerPanel.add(languageLabel, new GridBagConstraints(0, 4, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(20, 5, 0, 0), 0, 0));
        languageTextField = new JTextField(main.getPrefs().getString(Config.PREFS_LANGUAGE_FILE, EMPTY_STRING));
        centerPanel.add(languageTextField, new GridBagConstraints(1, 4, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(20, 0, 0, 5), 0, 0));

        JLabel fontLabel = new JLabel("Font file: ");
        centerPanel.add(fontLabel, new GridBagConstraints(0, 5, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
        fontTextField = new JTextField(main.getPrefs().getString(Config.PREFS_FONT_FILE, EMPTY_STRING));
        centerPanel.add(fontTextField, new GridBagConstraints(1, 5, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 5), 0, 0));

        JLabel downloadLabel = new JLabel("Download URL: ");
        centerPanel.add(downloadLabel, new GridBagConstraints(0, 6, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.NONE, new Insets(20, 5, 0, 0), 0, 0));
        downloadTextField = new JTextField(main.getPrefs().getString(Config.PREFS_DOWNLOAD_URL, EMPTY_STRING));
        centerPanel.add(downloadTextField, new GridBagConstraints(1, 6, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(20, 0, 0, 5), 0, 0));

        rootPane.add(centerPanel, BorderLayout.CENTER);

        okButton = new JButton("Ok");
        okButton.addActionListener(this);
        rootPane.add(okButton, BorderLayout.SOUTH);

        KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        rootPane.registerKeyboardAction(this, stroke, JComponent.WHEN_IN_FOCUSED_WINDOW);

        pack();

        Dimension maxSize = Utils.getDefaultSize();
        Dimension resultSize = centerPanel.getSize();
        resultSize.width = 400;

        if (resultSize.width < maxSize.width)
        {
            resultSize.width += 60;
        }
        if (resultSize.height < maxSize.height)
        {
            resultSize.height += 90;
        }

        okButton.requestFocusInWindow();
        setSize(resultSize);
        Utils.centerResizeWindow(this, false);
        setVisible(true);
        toFront();
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
        SettingsDialog.this.setVisible(false);
        SettingsDialog.this.dispose();

        if (e.getSource() == okButton)
        {
            checkNewPref(Config.PREFS_USB_PATH, usbPathTextField.getText());
            checkNewPref(Config.PREFS_ETH_PATH, ethPathTextField.getText());
            checkNewPref(Config.PREFS_PS2_IP, PS2IPTextField.getText());
            checkNewPref(Config.PREFS_APPS_FILE, appsTextField.getText());

            checkNewPref(Config.PREFS_FONT_FILE, fontTextField.getText());
            checkNewPref(Config.PREFS_LANGUAGE_FILE, languageTextField.getText());

            checkNewPref(Config.PREFS_DOWNLOAD_URL, downloadTextField.getText());

            main.init();
        }
    }

    private void checkNewPref(String key, String value)
    {
        value = value.trim();
        if (!value.equals(main.getPrefs().getString(key, EMPTY_STRING)))
        {
            if (value.equals(EMPTY_STRING))
            {
                main.getPrefs().remove(key);
            }
            else
            {
                main.getPrefs().setString(key, value);
            }
        }
    }
}
