package oplutil;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FilenameFilter;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import opl.Config;
import opl.RenderMan;
import opl.Themes;

public class SettingsPanel extends JPanel implements ActionListener
{
    private static final long serialVersionUID = 1L;

    class Theme
    {
        public String name;
        public String path;

        public Theme(String _name, String _path)
        {
            name = _name;
            path = _path;
        }

        @Override
        public String toString()
        {
            return name;
        }
    }

    private Main main;

    private JRadioButton NTSCRadio, PALRadio, StandardTVRadio, WideTVRadio;
    private JComboBox themeComboBox;
    private DefaultComboBoxModel themeModel;

    public SettingsPanel(Main parent)
    {
        main = parent;

        setBorder(BorderFactory.createLoweredBevelBorder());
        setLayout(new GridBagLayout());

        JLabel themeSelectionLabel = new JLabel("Themes: ");
        add(themeSelectionLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
        themeModel = new DefaultComboBoxModel();
        themeComboBox = new JComboBox(themeModel);
        themeComboBox.addActionListener(this);
        add(themeComboBox, new GridBagConstraints(1, 0, 3, 1, 1.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

        JLabel consoleModeLabel = new JLabel("PS2 type: ");
        add(consoleModeLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 5, 0, 0), 0, 0));
        NTSCRadio = new JRadioButton("NTSC");
        NTSCRadio.addActionListener(this);
        PALRadio = new JRadioButton("PAL");
        PALRadio.addActionListener(this);
        ButtonGroup consoleModeGroup = new ButtonGroup();
        consoleModeGroup.add(PALRadio);
        consoleModeGroup.add(NTSCRadio);
        if (Main.screenHeight == 512)
        {
            PALRadio.setSelected(true);
        }
        else
        {
            NTSCRadio.setSelected(true);
        }
        add(PALRadio, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        add(NTSCRadio, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        add(Box.createHorizontalGlue(), new GridBagConstraints(3, 1, 1, 1, 0.20, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));

        JLabel tvModeLabel = new JLabel("TV mode: ");
        add(tvModeLabel, new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        StandardTVRadio = new JRadioButton("4:3");
        StandardTVRadio.addActionListener(this);
        WideTVRadio = new JRadioButton("16:9");
        WideTVRadio.addActionListener(this);
        ButtonGroup tvModeGroup = new ButtonGroup();
        tvModeGroup.add(StandardTVRadio);
        tvModeGroup.add(WideTVRadio);
        StandardTVRadio.setSelected(true);
        add(StandardTVRadio, new GridBagConstraints(5, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        add(WideTVRadio, new GridBagConstraints(6, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        add(Box.createHorizontalGlue(), new GridBagConstraints(7, 1, 1, 1, 0.80, 0.0, GridBagConstraints.WEST,
                GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();
        if (source == PALRadio)
        {
            Main.screenHeight = 512;
            resetTheme();
        }
        else if (source == NTSCRadio)
        {
            Main.screenHeight = 448;
            resetTheme();
        }
        else if (source == StandardTVRadio)
        {
            RenderMan.rmSetAspectRatio(1, 1);
            main.getScreen().setWideScreen(false);
        }
        else if (source == WideTVRadio)
        {
            RenderMan.rmSetAspectRatio(3f / 4f, 1);
            main.getScreen().setWideScreen(true);
        }
        else if (source == themeComboBox)
        {
            resetTheme();
        }
    }

    private void resetTheme()
    {
        Theme selectedTheme = (Theme) themeModel.getSelectedItem();
        if (selectedTheme != null)
        {
            Themes newT = new Themes();
            newT.thmLoad(selectedTheme.path);
            if (main.setTheme(newT))
            {
                //MenuSys.menu.reset(); // TODO ??
            }

            main.getScreen().repaint();
        }
    }

    public void reloadThemes()
    {
        themeModel.removeAllElements();

        Theme internal = new Theme("<OPL>", null);

        themeModel.addElement(internal);

        addElements(main.getPrefs().getProperty(Config.PREFS_ETH_PATH));
        addElements(main.getPrefs().getProperty(Config.PREFS_USB_PATH));

        themeModel.setSelectedItem(internal);

        //themeModel.setSelectedItem(themeModel.getElementAt(1));
    }

    private void addElements(String path)
    {
        if (path != null)
        {
            File pathDir = new File(path + "/THM");
            if (pathDir.exists() && pathDir.isDirectory())
            {
                String[] themes = pathDir.list(new FilenameFilter()
                {
                    public boolean accept(File dir, String name)
                    {
                        if (name.startsWith("thm_"))
                        {
                            return true;
                        }
                        return false;
                    }
                });

                for (String themePath : themes)
                {
                    themeModel.addElement(new Theme(themePath.substring(4, themePath.length()), pathDir
                            .getAbsolutePath()
                            + "/" + themePath + "/"));
                }
            }
        }
    }
}
