package oplutil.device;

import java.awt.Image;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

import opl.Lang;
import opl.Textures;
import oplutil.Elem;
import oplutil.Main;

public class MenuApp extends MenuBase
{
    public MenuApp(Main main, int id, String key, int textId, int iconId)
    {
        super(main, id, key, false, textId, iconId);
    }
    
    @Override
    protected void populateItems(String newValue)
    {
        items.clear();
        File fileBuffer = new File(newValue);
        if (fileBuffer.exists())
        {
            BufferedReader reader = null;
            try
            {
                String line;
                int keyIndex;
                reader = new BufferedReader(new FileReader(fileBuffer));
                while ((line = reader.readLine()) != null)
                {
                    keyIndex = line.indexOf('=');
                    items.add(new ItemBase(line.substring(0, keyIndex), line.substring(keyIndex + 1, line.length())));
                }
            }
            catch(Exception ex)
            {
                ex.printStackTrace(System.err);
            }
            finally
            {
                if (reader != null)
                {
                    try
                    {
                        reader.close();
                    }
                    catch (Exception ex)
                    {}
                }
            }
        }
    }
    
    @Override
    public List<Elem> initHints()
    {
        List<Elem> result = new ArrayList<Elem>();

        result.add(new Elem(Lang._STR_SETTINGS, Textures.START_ICON));
        result.add(new Elem(Lang._STR_RUN, Textures.CROSS_ICON));
        result.add(new Elem(Lang._STR_RENAME, Textures.CIRCLE_ICON));
        result.add(new Elem(Lang._STR_DELETE, Textures.SQUARE_ICON));

        return result;
    }
    
    @Override
    public Image getArt(String path)
    {
        int pos = path.indexOf('/');
        if (pos == -1)
        {
            pos = path.indexOf(':');
        }
        if (pos != -1)
        {
            // We search on ever devices from fatest to slowest (HDD > ETH > USB)
            Image result = main.getMenu(Main.MENU_HDD).getArt(path.substring(pos + 1));
            if (result != null)
            {
                return result;
            }
            result = main.getMenu(Main.MENU_ETH).getArt(path.substring(pos + 1));
            if (result != null)
            {
                return result;
            }
            return main.getMenu(Main.MENU_USB).getArt(path.substring(pos + 1));
        }
        
        return null;
    }
}
