/*    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package util;

import java.awt.Component;
import java.awt.HeadlessException;
import java.io.File;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

public class FileChooser extends JFileChooser
{
    private static final long serialVersionUID = 1L;

    public FileChooser()
    {
        super();
        String currDir = System.getProperty("user.dir");
        if (currDir != null)
        {
            setCurrentDirectory(new File(currDir));
        }
        setMultiSelectionEnabled(true);
        setFileFilter(new ExtensionFileFilter(".iso"));
    }

    @Override
    protected JDialog createDialog(Component parent) throws HeadlessException
    {
        JDialog dialog = super.createDialog(parent);
        Utils.centerResizeWindow(dialog, true);
        return dialog;
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////////

    class ExtensionFileFilter extends FileFilter
    {
        String extension;

        public ExtensionFileFilter(String ext)
        {
            extension = ext.toUpperCase();
        }

        public boolean accept(File f)
        {
            if (!f.isDirectory() && !f.getName().toUpperCase().endsWith(extension))
            {
                return false;
            }
            return true;
        }

        public String getDescription()
        {
            return extension + " files";
        }
    }
}
