/*    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package util;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;

public class ImagePanel extends JPanel implements MouseListener
{
    private static final long serialVersionUID = 1L;
    
    private Image image;
    private Image rolloverImage;
    private boolean isRollOver = false;
    private ActionListener actionListener;

    //private Dimension preferredSize;

    public ImagePanel(String source)
    {
        this(source, null, null);
    }

    public ImagePanel(String source, String rollover, ActionListener action)
    {
        if (source != null)
        {
            image = Utils.getImage(source);
        }

        if (rollover != null)
        {
            rolloverImage = Utils.getImage(rollover);
            addMouseListener(this);
        }

        actionListener = action;
    }

    @Override
    public void paintComponent(Graphics g)
    {
        if (isRollOver)
        {
            g.drawImage(rolloverImage, 0, 0, null);
        }
        else if (image != null)
        {
            g.drawImage(image, 0, 0, null);
        }
    }

    @Override
    public void mouseEntered(MouseEvent evt)
    {
        isRollOver = true;
        repaint();
    }

    @Override
    public void mouseExited(MouseEvent evt)
    {
        isRollOver = false;
        repaint();
        getParent().repaint();
    }

    @Override
    public void mouseClicked(MouseEvent e)
    {}

    @Override
    public void mousePressed(MouseEvent e)
    {}

    @Override
    public void mouseReleased(MouseEvent e)
    {
        if (actionListener != null)
        {
            actionListener.actionPerformed(new ActionEvent(e.getSource(), e.getID(), null));
        }
    }

    /*@Override
    public Dimension getPreferredSize()
    {
        if (preferredSize == null)
        {
            int width = image.getWidth(this);
            int height = image.getHeight(this);
            if (width != -1 && height != -1)
            {
                preferredSize = new Dimension(width, height);
            }
            else
            {
                return new Dimension(200,100);
            }
        }
        return preferredSize;
    }*/
}
